@@ -133,7 +133,7 @@ setupCairoLayer (CompScreen *s,
     il->pixmap = XCreatePixmap (s->display->display, s->root, w, h, 32);
     if (!bindPixmapToTexture (s, &il->texture, il->pixmap, w, h, 32))
     {
-	compLogMessage (s->display, "resizeinfo", CompLogLevelWarn,
+	compLogMessage ("resizeinfo", CompLogLevelWarn,
 			"Bind Pixmap to Texture failure");
 	freeInfoLayer (s, il);
 	return;
@@ -145,7 +145,7 @@ setupCairoLayer (CompScreen *s,
 						       format, w, h);
     if (cairo_surface_status (il->surface) != CAIRO_STATUS_SUCCESS)
     {
-	compLogMessage (s->display, "resizeinfo", CompLogLevelWarn,
+	compLogMessage ("resizeinfo", CompLogLevelWarn,
 			"Could not create cairo layer surface,");
 	freeInfoLayer (s, il);
 	return;
@@ -154,7 +154,7 @@ setupCairoLayer (CompScreen *s,
     il->cr = cairo_create (il->surface);
     if (cairo_status (il->cr) != CAIRO_STATUS_SUCCESS)
     {
-	compLogMessage (s->display, "resizeinfo", CompLogLevelWarn,
+	compLogMessage ("resizeinfo", CompLogLevelWarn,
 			"Could not create cairo context");
 	freeInfoLayer (s, il);
 	return;
@@ -172,7 +172,7 @@ updateTextLayer (CompScreen *s)
     int                  widthInc, heightInc;
     int                  width, height, xv, yv;
     unsigned short       *color;
-    char                 *info;
+    char                 info[50];
     cairo_t              *cr;
     PangoLayout          *layout;
     PangoFontDescription *font;
@@ -204,8 +204,8 @@ updateTextLayer (CompScreen *s)
     cairo_restore (cr);
     cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
 
-    asprintf (&info, "%d x %d", xv, yv);
- 
+    snprintf (info, 50, "%d x %d", xv, yv);
+
     font = pango_font_description_new ();
     layout = pango_cairo_create_layout (is->textLayer.cr);
   
@@ -392,7 +392,7 @@ infoWindowGrabNotify (CompWindow   *w,
 
     INFO_SCREEN (s);
 
-    if (!is->pWindow && !(w->state & MAXIMIZE_STATE))
+    if ((!is->pWindow || !is->drawing) && !(w->state & MAXIMIZE_STATE))
     {
 	Bool showInfo;
 	showInfo = ((w->sizeHints.width_inc != 1) && 
@@ -403,7 +403,7 @@ infoWindowGrabNotify (CompWindow   *w,
 	{
 	    is->pWindow  = w;
 	    is->drawing  = TRUE;
-	    is->fadeTime = resizeinfoGetFadeTime (s->display);
+	    is->fadeTime = resizeinfoGetFadeTime (s->display) - is->fadeTime;
 
 	    is->resizeGeometry.x      = w->attrib.x;
 	    is->resizeGeometry.y      = w->attrib.y;
@@ -427,7 +427,7 @@ infoWindowUngrabNotify (CompWindow *w)
     if (w == is->pWindow)
     {
 	is->drawing = FALSE;
-	is->fadeTime = resizeinfoGetFadeTime (s->display);
+	is->fadeTime = resizeinfoGetFadeTime (s->display) - is->fadeTime;
 	damageScreen (s);
     }
 	
