@@ -23,10 +23,6 @@
  *
  */
 
-/* TODO list for this plugin
- * - is there a way to get icons larger than 96x96?
- */
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -34,7 +30,6 @@
 #include <sys/time.h>
 
 #include <X11/Xatom.h>
-#include <X11/cursorfont.h>
 
 #include <compiz-core.h>
 #include <compiz-text.h>
@@ -70,7 +65,7 @@ typedef struct _RingDrawSlot {
 typedef struct _RingDisplay {
     int		    screenPrivateIndex;
     HandleEventProc handleEvent;
-    Bool            textAvailable;
+    TextFunc        *textFunc;
 } RingDisplay;
 
 typedef struct _RingScreen {
@@ -95,8 +90,6 @@ typedef struct _RingScreen {
     int     rotAdjust;
     GLfloat rVelocity;
 
-    Cursor cursor;
-
     /* only used for sorting */
     CompWindow   **windows;
     RingDrawSlot *drawSlots;
@@ -104,13 +97,10 @@ typedef struct _RingScreen {
     int          nWindows;
 
     Window clientLeader;
-    Window selectedWindow;
+    CompWindow *selectedWindow;
 
     /* text display support */
-    CompTexture textTexture;
-    Pixmap      textPixmap;
-    int         textWidth;
-    int         textHeight;
+    CompTextData *textData;
 
     CompMatch match;
     CompMatch *currentMatch;
@@ -131,6 +121,8 @@ typedef struct _RingWindow {
 
 #define PI 3.1415926
 #define DIST_ROT (3600 / rs->nWindows)
+#define ICON_SIZE 64
+#define MAX_ICON_SIZE 256
 
 #define GET_RING_DISPLAY(d)				      \
     ((RingDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
@@ -157,6 +149,9 @@ isRingWin (CompWindow *w)
 {
     RING_SCREEN (w->screen);
 
+    if (w->destroyed)
+	return FALSE;
+
     if (w->attrib.override_redirect)
 	return FALSE;
 
@@ -209,23 +204,20 @@ isRingWin (CompWindow *w)
 static void 
 ringFreeWindowTitle (CompScreen *s)
 {
-    RING_SCREEN(s);
+    RING_SCREEN (s);
+    RING_DISPLAY (s->display);
 
-    if (!rs->textPixmap)
+    if (!rs->textData)
 	return;
 
-    releasePixmapFromTexture (s, &rs->textTexture);
-    initTexture (s, &rs->textTexture);
-    XFreePixmap (s->display->display, rs->textPixmap);
-    rs->textPixmap = None;
+    (rd->textFunc->finiTextData) (s, rs->textData);
+    rs->textData = NULL;
 }
 
 static void 
 ringRenderWindowTitle (CompScreen *s)
 {
-    CompTextAttrib tA;
-    int            stride;
-    void           *data;
+    CompTextAttrib attrib;
     int            ox1, ox2, oy1, oy2;
 
     RING_SCREEN (s);
@@ -233,7 +225,10 @@ ringRenderWindowTitle (CompScreen *s)
 
     ringFreeWindowTitle (s);
 
-    if (!rd->textAvailable)
+    if (!rd->textFunc)
+	return;
+
+    if (!rs->selectedWindow)
 	return;
 
     if (!ringGetWindowTitle (s))
@@ -242,74 +237,52 @@ ringRenderWindowTitle (CompScreen *s)
     getCurrentOutputExtents (s, &ox1, &oy1, &ox2, &oy2);
 
     /* 75% of the output device as maximum width */
-    tA.maxWidth = (ox2 - ox1) * 3 / 4;
-    tA.maxHeight = 100;
-    tA.screen = s;
-    tA.size = ringGetTitleFontSize (s);
-    tA.color[0] = ringGetTitleFontColorRed (s);
-    tA.color[1] = ringGetTitleFontColorGreen (s);
-    tA.color[2] = ringGetTitleFontColorBlue (s);
-    tA.color[3] = ringGetTitleFontColorAlpha (s);
-    tA.style = (ringGetTitleFontBold (s)) ?
-	       TEXT_STYLE_BOLD : TEXT_STYLE_NORMAL;
-    tA.style |= TEXT_STYLE_BACKGROUND;
-    tA.family = "Sans";
-    tA.ellipsize = TRUE;
-    tA.backgroundHMargin = 15;
-    tA.backgroundVMargin = 15;
-    tA.backgroundColor[0] = ringGetTitleBackColorRed (s);
-    tA.backgroundColor[1] = ringGetTitleBackColorGreen (s);
-    tA.backgroundColor[2] = ringGetTitleBackColorBlue (s);
-    tA.backgroundColor[3] = ringGetTitleBackColorAlpha (s);
+    attrib.maxWidth = (ox2 - ox1) * 3 / 4;
+    attrib.maxHeight = 100;
 
-    if (rs->type == RingTypeAll)
-	tA.renderMode = TextRenderWindowTitleWithViewport;
-    else
-	tA.renderMode = TextRenderWindowTitle;
+    attrib.size = ringGetTitleFontSize (s);
+    attrib.color[0] = ringGetTitleFontColorRed (s);
+    attrib.color[1] = ringGetTitleFontColorGreen (s);
+    attrib.color[2] = ringGetTitleFontColorBlue (s);
+    attrib.color[3] = ringGetTitleFontColorAlpha (s);
+    attrib.flags = CompTextFlagWithBackground | CompTextFlagEllipsized;
+    if (ringGetTitleFontBold (s))
+	attrib.flags |= CompTextFlagStyleBold;
+    attrib.family = "Sans";
+    attrib.bgHMargin = 15;
+    attrib.bgVMargin = 15;
+    attrib.bgColor[0] = ringGetTitleBackColorRed (s);
+    attrib.bgColor[1] = ringGetTitleBackColorGreen (s);
+    attrib.bgColor[2] = ringGetTitleBackColorBlue (s);
+    attrib.bgColor[3] = ringGetTitleBackColorAlpha (s);
 
-    tA.data = (void*)rs->selectedWindow;
 
-    initTexture (s, &rs->textTexture);
-
-    if ((*s->display->fileToImage) (s->display, TEXT_ID, (char *)&tA,
-			 	    &rs->textWidth, &rs->textHeight,
-				    &stride, &data))
-    {
-	rs->textPixmap = (Pixmap)data;
-	bindPixmapToTexture (s, &rs->textTexture, rs->textPixmap,
-			     rs->textWidth, rs->textHeight, 32);
-    }
-    else 
-    {
-	rs->textPixmap = None;
-	rs->textWidth  = 0;
-	rs->textHeight = 0;
-    }
+    rs->textData = (rd->textFunc->renderWindowTitle) (s,
+						      (rs->selectedWindow ?
+						       rs->selectedWindow->id :
+						       None),
+						      rs->type == RingTypeAll,
+						      &attrib);
 }
 
 static void
 ringDrawWindowTitle (CompScreen *s)
 {
-    GLboolean  wasBlend;
-    GLint      oldBlendSrc, oldBlendDst;
-    float      x, y, width, height;
+    float      x, y;
     int        ox1, ox2, oy1, oy2;
-    CompMatrix *m;
 
     RING_SCREEN (s);
-
-    width = rs->textWidth;
-    height = rs->textHeight;
+    RING_DISPLAY (s->display);
 
     getCurrentOutputExtents (s, &ox1, &oy1, &ox2, &oy2);
 
-    x = ox1 + ((ox2 - ox1) / 2) - (rs->textWidth / 2);
+    x = ox1 + ((ox2 - ox1) / 2) - ((int)rs->textData->width / 2);
 
     /* assign y (for the lower corner!) according to the setting */
     switch (ringGetTitleTextPlacement (s))
     {
 	case TitleTextPlacementCenteredOnScreen:
-	    y = oy1 + ((oy2 - oy1) / 2) + (height / 2);
+	    y = oy1 + ((oy2 - oy1) / 2) + ((int)rs->textData->height / 2);
 	    break;
 	case TitleTextPlacementAboveRing:
 	case TitleTextPlacementBelowRing:
@@ -319,7 +292,7 @@ ringDrawWindowTitle (CompScreen *s)
 
 	    	if (ringGetTitleTextPlacement (s) ==
 		    TitleTextPlacementAboveRing)
-    		    y = oy1 + workArea.y + height;
+    		    y = oy1 + workArea.y + (int)rs->textData->height;
 		else
 		    y = oy1 + workArea.y + workArea.height;
 	    }
@@ -329,44 +302,7 @@ ringDrawWindowTitle (CompScreen *s)
 	    break;
     }
 
-    x = floor (x);
-    y = floor (y);
-
-    glGetIntegerv (GL_BLEND_SRC, &oldBlendSrc);
-    glGetIntegerv (GL_BLEND_DST, &oldBlendDst);
-
-    wasBlend = glIsEnabled (GL_BLEND);
-    if (!wasBlend)
-	glEnable (GL_BLEND);
-
-    glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
-
-    glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-    glColor4f (1.0, 1.0, 1.0, 1.0);
-
-    enableTexture (s, &rs->textTexture,COMP_TEXTURE_FILTER_GOOD);
-
-    m = &rs->textTexture.matrix;
-
-    glBegin (GL_QUADS);
-
-    glTexCoord2f (COMP_TEX_COORD_X (m, 0), COMP_TEX_COORD_Y (m ,0));
-    glVertex2f (x, y - height);
-    glTexCoord2f (COMP_TEX_COORD_X (m, 0), COMP_TEX_COORD_Y (m, height));
-    glVertex2f (x, y);
-    glTexCoord2f (COMP_TEX_COORD_X (m, width), COMP_TEX_COORD_Y (m, height));
-    glVertex2f (x + width, y);
-    glTexCoord2f (COMP_TEX_COORD_X (m, width), COMP_TEX_COORD_Y (m, 0));
-    glVertex2f (x + width, y - height);
-
-    glEnd ();
-
-    disableTexture (s, &rs->textTexture);
-    glColor4usv (defaultColor);
-
-    if (!wasBlend)
-	glDisable (GL_BLEND);
-    glBlendFunc (oldBlendSrc, oldBlendDst);
+    (rd->textFunc->drawText) (s, rs->textData, floor (x), floor (y), 1.0f);
 }
 
 static Bool
@@ -427,7 +363,7 @@ ringPaintWindow (CompWindow		 *w,
     		fragment.brightness = (float) fragment.brightness * 
 		                      rw->slot->depthBrightness;
 
-		if (w->id != rs->selectedWindow)
+		if (w != rs->selectedWindow)
 		    fragment.opacity = (float)fragment.opacity *
 			               ringGetInactiveOpacity (s) / 100;
 	    }
@@ -457,7 +393,7 @@ ringPaintWindow (CompWindow		 *w,
 	{
 	    CompIcon *icon;
 
-	    icon = getWindowIcon (w, 96, 96);
+	    icon = getWindowIcon (w, MAX_ICON_SIZE, MAX_ICON_SIZE);
 	    if (!icon)
 		icon = w->screen->defaultIcon;
 
@@ -483,15 +419,28 @@ ringPaintWindow (CompWindow		 *w,
     		case OverlayIconNone:
 		case OverlayIconEmblem:
 		    scale = (rw->slot) ? rw->slot->depthScale : 1.0f;
+		    scale = MIN ((scale * ICON_SIZE / icon->width),
+				 (scale * ICON_SIZE / icon->height));
 		    break;
 		case OverlayIconBig:
 		default:
-		    /* only change opacity if not painting an
-		       icon for a minimized window */
 		    if (w->texture->pixmap)
+		    {
+			/* only change opacity if not painting an
+			   icon for a minimized window */
 			sAttrib.opacity /= 3;
-		    scale = MIN (((float) scaledWinWidth / icon->width),
-				 ((float) scaledWinHeight / icon->height));
+			scale = MIN (((float) scaledWinWidth / icon->width),
+				     ((float) scaledWinHeight / icon->height));
+		    }
+		    else
+		    {
+			scale =
+			    0.8f * ((rw->slot) ? rw->slot->depthScale : 1.0f);
+			scale = MIN ((scale * ringGetThumbWidth (s) /
+				      icon->width),
+				     (scale * ringGetThumbHeight (s) /
+				      icon->height));
+		    }
 		    break;
 		}
 
@@ -514,12 +463,8 @@ ringPaintWindow (CompWindow		 *w,
 		x += rw->tx;
 		y += rw->ty;
 
-		mask |= PAINT_WINDOW_BLEND_MASK;
-		
-		/* if we paint the icon for a minimized window, we need
-		   to force the usage of a good texture filter */
-		if (!w->texture->pixmap)
-		    mask |= PAINT_WINDOW_TRANSFORMED_MASK;
+		mask |=
+		    PAINT_WINDOW_BLEND_MASK | PAINT_WINDOW_TRANSFORMED_MASK;
 
 		iconReg.rects    = &iconReg.extents;
 		iconReg.numRects = 1;
@@ -543,11 +488,7 @@ ringPaintWindow (CompWindow		 *w,
 		    CompTransform  wTransform = *transform;
 
 		    if (!w->texture->pixmap)
-		    {
-			/* the fade plugin does weird things to
-			   w->paint.opacity, so better use the atom value */
-			sAttrib.opacity = w->opacity;
-		    }
+			sAttrib.opacity = w->paint.opacity;
 
 		    initFragmentAttrib (&fragment, &sAttrib);
 
@@ -593,6 +534,23 @@ ringLinearInterpolation (float valX,
     return (minY + (factor * (valX - minX)));
 }
 
+static void
+switchActivateEvent (CompScreen *s,
+		     Bool	activating)
+{
+    CompOption o[2];
+
+    o[0].type = CompOptionTypeInt;
+    o[0].name = "root";
+    o[0].value.i = s->root;
+
+    o[1].type = CompOptionTypeBool;
+    o[1].name = "active";
+    o[1].value.b = activating;
+
+    (*s->display->handleCompizEvent) (s->display, "ring", "activate", o, 2);
+}
+
 static int
 compareWindows (const void *elem1,
 		const void *elem2)
@@ -750,7 +708,7 @@ ringUpdateWindowList (CompScreen *s)
     rs->rotTarget = 0;
     for (i = 0; i < rs->nWindows; i++)
     {
-	if (rs->windows[i]->id == rs->selectedWindow)
+	if (rs->windows[i] == rs->selectedWindow)
 	    break;
 
 	rs->rotTarget += DIST_ROT;
@@ -796,7 +754,7 @@ switchToWindow (CompScreen *s,
 
     for (cur = 0; cur < rs->nWindows; cur++)
     {
-	if (rs->windows[cur]->id == rs->selectedWindow)
+	if (rs->windows[cur] == rs->selectedWindow)
 	    break;
     }
 
@@ -810,10 +768,10 @@ switchToWindow (CompScreen *s,
 
     if (w)
     {
-	Window old = rs->selectedWindow;
+	CompWindow *old = rs->selectedWindow;
 
-	rs->selectedWindow = w->id;
-	if (old != w->id)
+	rs->selectedWindow = w;
+	if (old != w)
 	{
 	    if (toNext)
 		rs->rotAdjust += DIST_ROT;
@@ -999,7 +957,7 @@ ringPaintOutput (CompScreen		  *s,
 
 	rs->paintingSwitcher = FALSE;
 
-	if (rs->textPixmap && (rs->state != RingStateIn))
+	if (rs->textData && rs->state != RingStateIn)
 	    ringDrawWindowTitle (s);
 	
 	glPopMatrix ();
@@ -1087,6 +1045,9 @@ ringDonePaintScreen (CompScreen *s)
 	    else if (rs->state == RingStateOut)
 		rs->state = RingStateSwitching;
 	}
+
+	if (rs->state == RingStateNone)
+	    switchActivateEvent (s, FALSE);
     }
 
     UNWRAP (rs, s, donePaintScreen);
@@ -1139,11 +1100,10 @@ ringTerminate (CompDisplay     *d,
     	    rs->state = RingStateIn;
     	    damageScreen (s);
 
-	    if (!(state & CompActionStateCancel) && rs->selectedWindow)
+	    if (!(state & CompActionStateCancel) &&
+		rs->selectedWindow && !rs->selectedWindow->destroyed)
 	    {
-		w = findWindowAtScreen (s, rs->selectedWindow);
-		if (w)
-		    sendWindowActivationRequest (s, w->id);
+		sendWindowActivationRequest (s, rs->selectedWindow->id);
 	    }
 	}
     }
@@ -1168,7 +1128,7 @@ ringInitiate (CompScreen      *s,
 
     RING_SCREEN (s);
 
-    if (otherScreenGrabExist (s, "ring", 0))
+    if (otherScreenGrabExist (s, "ring", NULL))
 	return FALSE;
 	   
     rs->currentMatch = ringGetWindowMatch (s);
@@ -1193,7 +1153,7 @@ ringInitiate (CompScreen      *s,
     if (!rs->grabIndex)
     {
 	if (ringGetSelectWithMouse (s))
-	    rs->grabIndex = pushScreenGrab (s, rs->cursor, "ring");
+	    rs->grabIndex = pushScreenGrab (s, s->normalCursor, "ring");
 	else
 	    rs->grabIndex = pushScreenGrab (s, s->invisibleCursor, "ring");
     }
@@ -1205,12 +1165,14 @@ ringInitiate (CompScreen      *s,
 	if (!ringCreateWindowList (s))
 	    return FALSE;
 
-    	rs->selectedWindow = rs->windows[0]->id;
+	rs->selectedWindow = rs->windows[0];
 	ringRenderWindowTitle (s);
 	rs->rotTarget = 0;
 
     	rs->moreAdjust = TRUE;
 	damageScreen (s);
+
+	switchActivateEvent (s, TRUE);
     }
 
     return TRUE;
@@ -1342,9 +1304,11 @@ ringPrevGroup (CompDisplay     *d,
 static void
 ringWindowSelectAt (CompScreen *s,
 		    int        x,
-		    int        y)
+		    int        y,
+		    Bool       terminate)
 {
-    int i;
+    int    i;
+    CompWindow *selected = NULL;
 
     RING_SCREEN (s);
 
@@ -1367,36 +1331,48 @@ ringWindowSelectAt (CompScreen *s,
 		(y <= (rw->ty + w->attrib.y + (w->attrib.height * rw->scale))))
 	    {
 		/* we have found one, select it */
-		rs->selectedWindow = w->id;
+		selected = w;
 		break;
 	    }
 	}
     }
 
-    if (i >= 0)
+    if (selected && terminate)
     {
 	CompOption o;
 
-    	o.type = CompOptionTypeInt;
-    	o.name = "root";
+	o.type = CompOptionTypeInt;
+	o.name = "root";
 	o.value.i = s->root;
 
+	rs->selectedWindow = selected;
+
 	ringTerminate (s->display, NULL, 0, &o, 1);
     }
+    else if (!terminate && (selected != rs->selectedWindow || !rs->textData))
+    {
+	if (!selected)
+	{
+	    ringFreeWindowTitle (s);
+	}
+	else
+	{
+	    rs->selectedWindow = selected;
+	    ringRenderWindowTitle (s);
+	}
+	damageScreen (s);
+    }
 }
 
 static void 
 ringWindowRemove (CompDisplay *d, 
-		  Window      id)
+		  CompWindow  *w)
 {
-    CompWindow *w;
-
-    w = findWindowAtDisplay (d, id);
     if (w)
     {
 	Bool   inList = FALSE;
 	int    j, i = 0;
-	Window selected;
+	CompWindow *selected;
 
 	RING_SCREEN (w->screen);
 
@@ -1410,18 +1386,19 @@ ringWindowRemove (CompDisplay *d,
 
 	while (i < rs->nWindows)
 	{
-    	    if (w->id == rs->windows[i]->id)
+	    if (w == rs->windows[i])
 	    {
 		inList = TRUE;
 
-		if (w->id == selected)
+		if (w == selected)
 		{
 		    if (i < (rs->nWindows - 1))
-			selected = rs->windows[i + 1]->id;
-    		    else
-			selected = rs->windows[0]->id;
+			selected = rs->windows[i + 1];
+		    else
+			selected = rs->windows[0];
 
 		    rs->selectedWindow = selected;
+		    ringRenderWindowTitle (w->screen);
 		}
 
 		rs->nWindows--;
@@ -1449,7 +1426,9 @@ ringWindowRemove (CompDisplay *d,
 	    return;
 	}
 
-	if (!rs->grabIndex)
+	// Let the window list be updated to avoid crash
+	// when a window is closed while ending (RingStateIn).
+	if (!rs->grabIndex && rs->state != RingStateIn)
 	    return;
 
 	if (ringUpdateWindowList (w->screen))
@@ -1465,6 +1444,19 @@ static void
 ringHandleEvent (CompDisplay *d,
 		 XEvent      *event)
 {
+    CompScreen *s;
+    CompWindow *w = NULL;
+
+    switch (event->type) {
+    case DestroyNotify:
+	/* We need to get the CompWindow * for event->xdestroywindow.window
+	   here because in the (*d->handleEvent) call below, that CompWindow's
+	   id will become 1, so findWindowAtDisplay won't be able to find the
+	   CompWindow after that. */
+	w = findWindowAtDisplay (d, event->xdestroywindow.window);
+	break;
+    }
+
     RING_DISPLAY (d);
 
     UNWRAP (rd, d, handleEvent);
@@ -1480,7 +1472,7 @@ ringHandleEvent (CompDisplay *d,
 	    if (w)
 	    {
     		RING_SCREEN (w->screen);
-    		if (rs->grabIndex && (w->id == rs->selectedWindow))
+    		if (rs->grabIndex && (w == rs->selectedWindow))
     		{
     		    ringRenderWindowTitle (w->screen);
     		    damageScreen (w->screen);
@@ -1491,7 +1483,6 @@ ringHandleEvent (CompDisplay *d,
     case ButtonPress:
 	if (event->xbutton.button == Button1)
 	{
-	    CompScreen *s;
 	    s = findScreenAtDisplay (d, event->xbutton.root);
 	    if (s)
 	    {
@@ -1500,15 +1491,29 @@ ringHandleEvent (CompDisplay *d,
     		if (rs->grabIndex)
     		    ringWindowSelectAt (s, 
 					event->xbutton.x_root, 
-					event->xbutton.y_root);
+					event->xbutton.y_root,
+					TRUE);
 	    }
 	}
 	break;
+    case MotionNotify:
+	s = findScreenAtDisplay (d, event->xmotion.root);
+	if (s)
+	{
+	    RING_SCREEN (s);
+
+	    if (rs->grabIndex)
+		ringWindowSelectAt (s,
+				    event->xmotion.x_root,
+				    event->xmotion.y_root,
+				    FALSE);
+	}
     case UnmapNotify:
-	ringWindowRemove (d, event->xunmap.window);
+	w = findWindowAtDisplay (d, event->xunmap.window);
+	ringWindowRemove (d, w);
 	break;
     case DestroyNotify:
-	ringWindowRemove (d, event->xdestroywindow.window);
+	ringWindowRemove (d, w);
 	break;
     }
 }
@@ -1566,6 +1571,7 @@ ringInitDisplay (CompPlugin  *p,
 		 CompDisplay *d)
 {
     RingDisplay *rd;
+    int         index;
 
     if (!checkPluginABI ("core", CORE_ABIVERSION))
 	return FALSE;
@@ -1581,10 +1587,17 @@ ringInitDisplay (CompPlugin  *p,
 	return FALSE;
     }
 
-    rd->textAvailable = checkPluginABI ("text", TEXT_ABIVERSION);
-    if (!rd->textAvailable)
-	compLogMessage (d, "ring", CompLogLevelWarn,
+    if (checkPluginABI ("text", TEXT_ABIVERSION) &&
+	getPluginDisplayIndex (d, "text", &index))
+    {
+	rd->textFunc = d->base.privates[index].ptr;
+    }
+    else
+    {
+	compLogMessage ("ring", CompLogLevelWarn,
 			"No compatible text plugin found.");
+	rd->textFunc = NULL;
+    }
 
     ringSetNextKeyInitiate (d, ringNext);
     ringSetNextKeyTerminate (d, ringTerminate);
@@ -1661,7 +1674,7 @@ ringInitScreen (CompPlugin *p,
 
     rs->paintingSwitcher = FALSE;
 
-    rs->selectedWindow = None;
+    rs->selectedWindow = NULL;
 
     rs->moreAdjust   = FALSE;
     rs->rotateAdjust = FALSE;
@@ -1669,7 +1682,7 @@ ringInitScreen (CompPlugin *p,
     rs->rotAdjust = 0;
     rs->rVelocity = 0;
 
-    rs->textPixmap = None;
+    rs->textData = NULL;
 
     matchInit (&rs->match);
 
@@ -1679,8 +1692,6 @@ ringInitScreen (CompPlugin *p,
     WRAP (rs, s, paintWindow, ringPaintWindow);
     WRAP (rs, s, damageWindowRect, ringDamageWindowRect);
 
-    rs->cursor = XCreateFontCursor (s->display->display, XC_left_ptr);
-
     s->base.privates[rd->screenPrivateIndex].ptr = rs;
 
     return TRUE;
@@ -1704,9 +1715,6 @@ ringFiniScreen (CompPlugin *p,
 
     ringFreeWindowTitle (s);
 
-    if (rs->cursor)
-	XFreeCursor (s->display->display, rs->cursor);
-
     if (rs->windows)
 	free (rs->windows);
 
