@@ -67,6 +67,7 @@ typedef struct _WorkaroundsWindow {
     Bool madeSticky;
     Bool madeFullscreen;
     Bool isFullscreen;
+    Bool madeDemandAttention;
 } WorkaroundsWindow;
 
 #define GET_WORKAROUNDS_DISPLAY(d) \
@@ -197,6 +198,9 @@ workaroundsPaintScreen (CompScreen   *s,
 
     currentScreen = s;
 
+    if (workaroundsGetForceGlxSync (s->display))
+	glXWaitX ();
+
     UNWRAP (ws, s, paintScreen);
     (*s->paintScreen) (s, outputs, numOutputs, mask);
     WRAP (ws, s, paintScreen, workaroundsPaintScreen);
@@ -270,6 +274,27 @@ workaroundsUpdateSticky (CompWindow *w)
 	workaroundsRemoveSticky (w);
 }
 
+static void
+updateUrgencyState (CompWindow *w)
+{
+    Bool urgent;
+
+    WORKAROUNDS_WINDOW (w);
+
+    urgent = (w->hints && (w->hints->flags & XUrgencyHint));
+
+    if (urgent)
+    {
+	ww->madeDemandAttention = TRUE;
+	changeWindowState (w, w->state | CompWindowStateDemandsAttentionMask);
+    }
+    else if (ww->madeDemandAttention)
+    {
+	ww->madeDemandAttention = FALSE;
+	changeWindowState (w, w->state & ~CompWindowStateDemandsAttentionMask);
+    }
+}
+
 static void
 workaroundsGetAllowedActionsForWindow (CompWindow   *w,
 				       unsigned int *setActions,
@@ -588,6 +613,15 @@ workaroundsHandleEvent (CompDisplay *d,
 	    if (w)
 		workaroundsDoFixes (w);
 	}
+	else if (event->xproperty.atom == XA_WM_HINTS)
+	{
+	    if (workaroundsGetConvertUrgency (d))
+	    {
+		w = findWindowAtDisplay (d, event->xproperty.window);
+		if (w)
+		    updateUrgencyState (w);
+	    }
+	}
 	break;
     default:
 	break;
@@ -728,6 +762,7 @@ workaroundsInitWindow (CompPlugin *plugin, CompWindow *w)
     ww->adjustedWinType = FALSE;
     ww->isFullscreen = FALSE;
     ww->madeFullscreen = FALSE;
+    ww->madeDemandAttention = FALSE;
 
     w->base.privates[ws->windowPrivateIndex].ptr = ww;
 
@@ -739,16 +774,19 @@ workaroundsFiniWindow (CompPlugin *plugin, CompWindow *w)
 {
     WORKAROUNDS_WINDOW (w);
 
-    if (ww->adjustedWinType)
+    if (!w->destroyed)
     {
-	w->wmType = getWindowType (w->screen->display, w->id);
-	recalcWindowType (w);
-	recalcWindowActions (w);
-    }
+	if (ww->adjustedWinType)
+	{
+	    w->wmType = getWindowType (w->screen->display, w->id);
+	    recalcWindowType (w);
+	    recalcWindowActions (w);
+	}
 
-    if (w->state & CompWindowStateStickyMask && ww->madeSticky)
-	setWindowState (w->screen->display,
-			w->state & ~CompWindowStateStickyMask, w->id);
+	if (w->state & CompWindowStateStickyMask && ww->madeSticky)
+	    setWindowState (w->screen->display,
+			    w->state & ~CompWindowStateStickyMask, w->id);
+    }
 
     free (ww);
 }
